"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Sub = void 0;
const http_1 = require("http");
const lodash_merge_1 = __importDefault(require("lodash.merge"));
const querystring_1 = require("querystring");
const group_1 = require("../../../components/group/group");
const job_file_storage_1 = require("../../../libs/queue-jobs/job-file-storage");
const jobs_manager_1 = require("../../../libs/queue-jobs/jobs-manager");
const sub_job_1 = require("./sub.job");
class Sub extends group_1.Group {
    address = '0.0.0.0:8811';
    queue;
    jobsManager;
    server;
    constructor() {
        super();
        this.$$ignoreEvalProps.push('server', 'queueRunner');
    }
    async init(props) {
        await super.init(props);
        Object.assign(this, props);
    }
    async exec() {
        if (this.queue?.file) {
            this.jobsManager = new jobs_manager_1.JobsManager(this.logger, {
                concurrent: this.queue.concurrent || 1,
                jobStorage: this.queue.file ? new job_file_storage_1.JobFileStorage(this.logger, this.scene.getPath(this.queue.file), this.queue.password) : undefined,
            });
        }
        this.server = (0, http_1.createServer)((req, res) => this.handleRequest(req, res));
        await new Promise((resolve, reject) => {
            this.server?.on('error', reject);
            this.server?.on('close', () => {
                this.jobsManager ? this.jobsManager.stop() : resolve(undefined);
            });
            const [host, port] = this.address.trim().split(':');
            this.server?.listen(+port, host, async () => {
                this.logger.debug('http/jobs listened at %s', this.address);
                if (this.jobsManager) {
                    try {
                        await this.jobsManager.start();
                        resolve(undefined);
                    }
                    catch (err) {
                        reject(err);
                    }
                }
            });
        });
        this.server = undefined;
        return [];
    }
    async exit() {
        if (!this.server?.listening)
            return;
        this.server?.close();
    }
    async dispose() {
        await this.exit();
    }
    getRequestBody(req) {
        return new Promise((resolve, reject) => {
            const chunks = [];
            req.on("data", (chunk) => chunks.push(chunk));
            req.on('error', reject);
            req.on("end", () => {
                const data = Buffer.concat(chunks).toString();
                resolve(data ? JSON.parse(data) : data);
            });
        });
    }
    async handleRequest(req, res) {
        this.logger.debug(`%s %s \t%s`, '⥃', req.method, req.url);
        try {
            let data = {};
            const qstr = req.url?.split('?')[1];
            if (qstr) {
                data = (0, querystring_1.parse)(qstr);
            }
            const requestBody = await this.getRequestBody(req);
            if (requestBody) {
                (0, lodash_merge_1.default)(data, requestBody);
            }
            if (this.jobsManager) {
                await this.jobsManager?.add(new sub_job_1.SubJob(super.exec.bind(this), { jobData: data }));
            }
            else {
                await super.exec({ jobData: data });
            }
        }
        finally {
            res.statusCode = 201;
            res.end();
        }
    }
}
exports.Sub = Sub;
//# sourceMappingURL=sub.js.map