"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JobsManager = void 0;
class JobsManager {
    logger;
    concurrent = 1;
    startProms;
    jobs = new Array();
    queueJobs = new Array();
    runningJobs = new Set();
    jobHandler;
    jobStorage;
    error;
    constructor(logger, opts) {
        this.logger = logger;
        Object.assign(this, opts);
    }
    async add(job) {
        this.jobs.push(job);
        if (this.jobStorage)
            await this.jobStorage.update(this.jobs);
        this.queueJobs.push(job);
        this.pullJobToRun();
    }
    pullJobToRun() {
        if (!this.startProms)
            return;
        if (!this.concurrent) {
            if (!this.runningJobs.size) {
                this.error ? this.startProms.reject(this.error) : this.startProms.resolve();
            }
            return;
        }
        if (this.queueJobs.length && this.runningJobs.size < this.concurrent) {
            const job = this.queueJobs.shift();
            if (job) {
                this.logger.debug('Pull a job to run   \t%j', job);
                setImmediate(async (job) => {
                    let jobError;
                    try {
                        if (this.jobHandler?.onJobRun)
                            await this.jobHandler.onJobRun(job);
                        await job.jobExecute();
                        this.logger.debug('A job is successful     \t%j', job);
                        if (this.jobHandler?.onJobSuccess)
                            await this.jobHandler.onJobSuccess(job);
                        this.jobs.splice(this.jobs.indexOf(job), 1);
                        if (this.jobStorage)
                            await this.jobStorage.update(this.jobs);
                    }
                    catch (err) {
                        this.logger.debug('A job is failed         \t%j', { jobError, job });
                        jobError = err;
                        this.queueJobs.splice(0, 0, job);
                        try {
                            if (this.jobHandler?.onJobFailure)
                                await this.jobHandler.onJobFailure(jobError, job);
                        }
                        catch (error) {
                            this.logger.debug('Jobs manager is stoped\t%j', { jobError, job, error });
                            this.concurrent = 0;
                            this.error = error;
                        }
                    }
                    finally {
                        this.runningJobs.delete(job);
                        if (this.jobHandler?.onJobDone)
                            await this.jobHandler.onJobDone(job);
                    }
                    this.pullJobToRun();
                }, job);
                this.runningJobs.add(job);
            }
        }
    }
    async start() {
        if (this.jobStorage) {
            const jobs = await this.jobStorage.init();
            jobs.forEach((job) => this.add(job));
        }
        return new Promise((resolve, reject) => {
            this.startProms = { resolve, reject };
            for (let i = 0; i < this.concurrent; i++) {
                this.pullJobToRun();
            }
        });
    }
    stop() {
        this.concurrent = 0;
        this.pullJobToRun();
    }
}
exports.JobsManager = JobsManager;
//# sourceMappingURL=jobs-manager.js.map